@@ -6,11 +6,8 @@ from django.contrib.admin.sites import AlreadyRegistered
 from django.contrib.sites.models import Site
 from django.http import HttpResponseRedirect
 from django.urls import reverse
+from django_audit_fields.admin import audit_fields, audit_fieldset_tuple
 from edc_label import Label
-from edc_model_admin.model_admin_audit_fields_mixin import (
-    audit_fields,
-    audit_fieldset_tuple,
-)
 from edc_protocol import Protocol
 
 from .admin_site import edc_randomization_admin
@@ -61,24 +58,27 @@ class RandomizationListModelAdmin(admin.ModelAdmin):
 
     search_fields = ("subject_identifier", "sid")
 
-    readonly_fields = (
-        "subject_identifier",
-        "sid",
-        "site_name",
-        "assignment",
-        "allocated",
-        "allocated_user",
-        "allocated_datetime",
-        "allocated_site",
-        "randomizer_name",
-    ) + audit_fields
-
     def get_fieldsets(self, request, obj=None):
         return (
             (None, {"fields": self.get_fieldnames(request)}),
             audit_fieldset_tuple,
         )
 
+    def get_readonly_fields(self, request, obj=None):
+        readonly_fields = super().get_readonly_fields(request, obj=obj)
+        readonly_fields += (
+            "subject_identifier",
+            "sid",
+            "site_name",
+            "assignment",
+            "allocated",
+            "allocated_user",
+            "allocated_datetime",
+            "allocated_site",
+            "randomizer_name",
+        ) + audit_fields
+        return tuple(set(readonly_fields))
+
     def get_queryset(self, request):
         """
         Filter the changelist to show for this site_name only.
